//*******************************************************************
// Stepper Motor Driver Demo and Testprogram for
//
//    Allegro A3977 MicroStepping Controller
//
// This program shows how simple a driver for this controller is.
// The controller does microsteps, 8 per phase
// As the stepping power stage the E-LAB MillAxrDriver board is used
// The E-LAB I2CkeyLed board is used for start movements
// The AVRco Stepper Driver is operated in UserPort mode.
// Please note that the steprate is doubled in this mode
// The E-LAB EvaBoard-I is used as the controlling device
// The test board is connected to the PORT_A header
//*******************************************************************

program StepMicro;

{$NOSHADOW}

Device = mega128;

Import SysTick, StepPort, LCDmultiPort, BeepPort, TWImaster;

From system import longword;

Define
  ProcClock      = 16000000;       {Hertz}
  SysTick        = 10;             {msec}
  StackSize      = $0070, iData;
  FrameSize      = $0100, iData;
  TWIpresc       = TWI_BR400;
  LCDmultiPort   = I2C_TWI;
  LCDrows_M      = 4;              {rows}
  LCDcolumns_M   = 16;             {columns per line}
  LCDtype_M      = 44780;
  BeepPort       = PortG, 4;
  StepMinFreq    = 100;
  StepMaxFreq    = 10000;
  StepType       = UserPort;


Implementation

{$IDATA}
{--------------------------------------------------------------}
{ Type Declarations }
type
  tLeds          = (Led0, Led1, Led2, Led3, Led4, Led5, Led6, Led7);
  tLedSet        = bitset of tLeds;

  tKeys          = (Key0, Key1, Key2, Key3, Key4, Key5, Key6, Key7);
  tKeySet        = bitset of tKeys;

  tStepPwr       = (pwrOff, pwrLow, pwrMed, pwrHigh);

{--------------------------------------------------------------}
{ Const Declarations }
const
  I2CKeyLed      : byte      = $26;
  I2CKey_RD      : byte      = 0;
  I2CKey_WR      : byte      = 2;
  I2CKey_DDR     : byte      = 6;
  I2CKey_Pol     : byte      = 4;          // polarity setting for input register only
  I2CLed_RD      : byte      = 1;
  I2CLed_WR      : byte      = 3;
  I2CLed_DDR     : byte      = 7;
  I2CLed_Pol     : byte      = 5;          // polarity setting for input register only

  LedAll         : tLedSet   = [tLeds];

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
VAR
  StepOut[@PortA]            : byte;
  StepDDR[@DDRA]             : byte;
  StepStat[@PinA]            : byte;
  StepPulse[@StepOut, 0]     : bit;
  StepDIR[@StepOut, 1]       : bit;
  StepHome[@StepStat, 2]     : bit;
  StepLimit[@StepStat, 3]    : bit;
  StepRefLo[@StepOut, 5]     : bit;
  StepRefHi[@StepOut, 4]     : bit;

  idx, i         : byte;
  led            : tLeds;
  Key            : tKeys;
  keysNew        : tKeySet;
  keysOld        : tKeySet;
  st             : string[16];

{--------------------------------------------------------------}
{ functions }
UserDevice StepperIOS(cw : boolean);
begin
  StepDir:= cw;
  StepPulse:= true;
  sDelay(20);
  StepPulse:= false;
end;

procedure KeyLedInit;
begin
  TWIout(I2CKeyLed, I2CLed_DDR, $00);        // all output
  TWIout(I2CKeyLed, I2CKey_Pol, $FF);        // all keys inverted
end;

procedure SetLedAll(leds : tLedSet);
begin
  TWIout(I2CKeyLed, I2CLed_WR, (not leds));
end;

procedure SetLedOn(led : tLeds);
var
  ld             : tLedSet;
begin
  TWIout(I2CKeyLed, I2CLed_RD);
  TWIinp(I2CKeyLed, ld);
  excl(ld, [led]);
  TWIout(I2CKeyLed, I2CLed_WR, ld);
end;

procedure SetLedOff(led : tLeds);
var
  ld             : tLedSet;
begin
  TWIout(I2CKeyLed, I2CLed_RD);
  TWIinp(I2CKeyLed, ld);
  incl(ld, [led]);
  TWIout(I2CKeyLed, I2CLed_WR, ld);
end;

function KeyPressed : boolean;
begin
  KeysOld:= KeysNew;
  TWIout(I2CKeyLed, I2CKey_RD);
  TWIinp(I2CKeyLed, KeysNew);
  // a little bit debouncing
  if KeysNew <> KeysOld then
    // is bouncing
    return(false);
  else
    return(KeysNew <> []);
  endif;
end;

function GetKeySet : tKeySet;
begin
  KeysOld:= KeysNew;
  TWIout(I2CKeyLed, I2CKey_RD);
  TWIinp(I2CKeyLed, KeysNew);
  // a little bit debouncing
  if KeysNew <> KeysOld then
    // is bouncing
    return([]);
  else
    return(KeysNew);
  endif;
end;

function GetKey : tKeys;
var
  keys           : tKeySet;
begin
  keys:= GetKeySet;
  case keys of
    [key0] : return(key0);
           |
    [key1] : return(key1);
           |
    [key2] : return(key2);
           |
    [key3] : return(key3);
           |
    [key4] : return(key4);
           |
    [key5] : return(key5);
           |
    [key6] : return(key6);
           |
    [key7] : return(key7);
           |
  else
    return(tKeys($ff));              // key not found, none active, more than one pressed
  endcase;
end;

procedure StepPortInit;
begin
  StepOut:= 0;
  StepDDR:= %00110011;
end;

procedure StepperPwr(pwr : tStepPwr);
begin
  case pwr of
    pwrOff  : StepRefLo:= 0;
              StepRefHi:= 0;
            |
    pwrLow  : StepRefLo:= 1;
              StepRefHi:= 0;
            |
    pwrMed  : StepRefLo:= 0;
              StepRefHi:= 1;
            |
    pwrHigh : StepRefLo:= 1;
              StepRefHi:= 1;
            |
  endcase;
end;


{--------------------------------------------------------------}
{ Main Program }
{$IDATA}
begin
  StepPortInit;
  EnableInts;

  LCDsetup_M(LCD_m1);
  LCDcursor_M(LCD_m1, true, false);
  Write(LCDOut_M,  'E-LAB EvaBoard');
  LCDxy_M(LCD_m1, 0, 1);
  Write(LCDOut_M,  'StepMicro Demo');
  BeepOut(2000, 20);
  mDelay(1000);

  LCDxy_M(LCD_m1, 0, 2);
  if TWIstat(I2CKeyLed) then
    Write(LCDOut_M,  'I2CkeyLed found');
    KeyLedInit;
    SetLedAll(LedAll);
    mDelay(1000);
    SetLedAll([]);
    BeepOut(2000, 5);
  else
    Write(LCDOut_M,  'I2CkeyLed failed');
    BeepSiren(0, 3);
  endif;

  loop
    SetLedAll([]);
    if KeyPressed then
      LCDxy_M(LCD_m1, 0, 3);
      BeepClick;
      case GetKey of
        Key0 : Write(LCDOut_M,  'Key 0');
               SetLedOn(Led0);
               While GetKey = Key0 do
                 StepperPwr(pwrMed);
                 StepOneCW;
                 mDelay(500);
               endwhile;
             |
        Key1 : Write(LCDOut_M,  'Key 1');
               SetLedOn(Led1);
               While GetKey = Key1 do
                 StepperPwr(pwrHigh);
                 StepOneCCW;
                 mDelay(1);
               endwhile;
             |
        Key2 : Write(LCDOut_M,  'Key 2');
               SetLedOn(Led2);
               StepperPwr(pwrHigh);
               StepStartFreq:=   100;              (*  700Hz *)
               StepEndFreq:=    6000;              (*  6kHz *)
               StepCount:=      9000;              (*  5000 steps *)
               StepAccValue:=   3000;              (*  3000Hz/sec *)
               StepDestCW;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
        Key3 : Write(LCDOut_M,  'Key 3');
               SetLedOn(Led3);
               StepperPwr(pwrHigh);
               StepStartFreq:=   500;              (*  500Hz *)
               StepEndFreq:=    8000;              (*   8kHz *)
               StepCount:=      9000;              (*  100000 steps *)
               StepAccValue:=   5000;              (*  5000Hz/sec *)
               StepDestCCW;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
        Key4 : Write(LCDOut_M,  'Key 4');
               SetLedOn(Led4);
               StepperPwr(pwrHigh);
               StepStartFreq:=   250;              (*  250Hz *)
               StepEndFreq:=    4000;              (*  4000Hz *)
               StepAccValue:=   2000;              (*  2000Hz/sec *)
               StepRampCW;
               repeat
               until not KeyPressed;
               SetLedOff(Led0);
               StepRampStop;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
        Key5 : Write(LCDOut_M,  'Key 5');
               SetLedOn(Led5);
               StepperPwr(pwrHigh);
               StepStartFreq:=   100;               (*  100Hz *)
               StepEndFreq:=    6000;               (*  6kHz *)
               StepAccValue:=   4000;               (*  4000Hz/sec *)
               StepRampCCW;
               repeat
               until not KeyPressed;
               StepRampStop;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
        Key6 : Write(LCDOut_M,  'Key 6');
               SetLedOn(Led6);
               StepperPwr(pwrHigh);
               StepStartFreq:=   50;                (*  50Hz *)
               StepEndFreq:=    2000;               (*  2kHz *)
               StepAccValue:=   500;                (*  500Hz/sec *)
               StepRampCW;
               repeat
               until not KeyPressed;
               StepRampStop;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
        Key7 : Write(LCDOut_M,  'Key 7');
               SetLedOn(Led7);
               StepperPwr(pwrHigh);
               StepStartFreq:=   50;                (*  50Hz *)
               StepEndFreq:=    2500;               (*  1kHz *)
               StepAccValue:=   500;                (*  500Hz/sec *)
               StepRampCCW;
               repeat
               until not KeyPressed;
               StepRampStop;
               repeat
               until StepMode = StepStop;
               StepperPwr(pwrLow);
             |
      else
        mDelay(150);
      endcase;
    endif;
    LCDxy_M(LCD_m1, 7, 3);
    Write(LCDOut_M,  LongToStr(StepCount:8));
    StepperPwr(pwrLow);
  endloop;
end StepMicro.


